/*
	*
	* This file is a part of IonShell.
	* IonShell is the Desktop Shell App for Ion DE
	* Copyright 2020 CuboCore Group
	*

	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* (at your option) any later version.
	*

	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*

	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/

#pragma once

#include <QObject>
#include <QMargins>
#include <QSize>

class QWindow;

#include "DesQLayerShell.hpp"
#include "qwayland-wlr-layer-shell-unstable-v1.h"

class DesQLayerShellPrivate : public QtWayland::zwlr_layer_shell_v1 {

	public:
		DesQLayerShellPrivate( struct ::zwlr_layer_shell_v1 *lShell, DesQLayerShell *ptr );
		struct ::zwlr_layer_surface_v1 *getDesQLayerSurface(QWindow *window, DesQLayerShell::LayerType layer, const QString &layer_namespace);

	private:
		struct ::zwlr_layer_shell_v1 *shellPtr;

	protected:
		Q_DISABLE_COPY( DesQLayerShellPrivate )
		Q_DECLARE_PUBLIC( DesQLayerShell )

		DesQLayerShell *q_ptr;
};

class DesQLayerSurfacePrivate : public QtWayland::zwlr_layer_surface_v1 {

	public:
		explicit DesQLayerSurfacePrivate( QWindow *window, DesQLayerShell *lShell, DesQLayerShell::LayerType type, DesQLayerSurface *ptr );

		void setLayer( DesQLayerShell::LayerType type );

		void apply();

	protected:
		void zwlr_layer_surface_v1_configure(uint32_t serial, uint32_t width, uint32_t height) override;
		void zwlr_layer_surface_v1_closed() override;

		Q_DISABLE_COPY( DesQLayerSurfacePrivate )
		Q_DECLARE_PUBLIC( DesQLayerSurface )

		DesQLayerSurface *q_ptr;

	private:
		QWindow *m_window;

		QSize m_surfaceSize;
		DesQLayerSurface::SurfaceAnchors m_anchors;
		int m_exclusiveZone;
		QMargins m_margins;
		uint m_keyboardInteractivity;

		QSize m_surfacePendingSize;
		bool isConfigured;
};
